import sys
import os
import re
import glob
import time
import subprocess

import configparser
import json

from PyQt5.QtWidgets import QApplication, QWidget, QMainWindow, QLabel, QGridLayout, QPushButton, QVBoxLayout, QFrame, QFileDialog, QInputDialog #, QDialog, QDialogButtonBox
from PyQt5.QtWidgets import QMessageBox, QAction, QHBoxLayout, QTabWidget, QGroupBox, QRadioButton, QListWidget, QDateTimeEdit, QStatusBar
from PyQt5.QtCore    import Qt, QRect, QDate, QDateTime, QFileInfo
from PyQt5.QtGui     import QPalette, QColor, QFont
from PyQt5 import QtCore, QtWidgets

import iws_defs
import HkMonitorWindow

                    
###class MockIWSCCWindow(QWidget):
class MockIWSCCWindow(QMainWindow):
    def __init__(self, title, *args, **kwargs):
        super(MockIWSCCWindow, self).__init__(*args, **kwargs)

        # Set time format to UTC
        os.environ["TZ"] = "UTC"

        self.setWindowTitle(title)

        self.name = 'MainWindow'

        self.mode = ''
        self.connection = 'NoConn'
        self.protocol = 'CCS'
#        self.connected = False
        self.connected = True # TEMP
        self.export_mounted = False
        self.tail_process = 0 # or maybe None

        self.windows = []
        self.wdict = {}

        self.win = 0

        self.subWindow = None


        #self.configuration = 'packet_defaults.json'
        self.configuration = ''
        self.source = ''
        self.current_catalog_file = ''

        self.start_datetime = ''
        self.end_datetime = ''

        #self.config = configparser.ConfigParser(allow_no_value=True)
        self.config = configparser.ConfigParser()
        self.config.read(iws_defs.config_directory + 'playback.ini')
        self.configs = self.config['CONFIGS'].values()
        self.sources = self.config['SOURCES'].values()

        # Take a stupid copy of the sources configuration because pyqt5 is stupid
        self.srcs = []
        for s in self.sources:
            self.srcs.append(s)

        # Get a list of configuration files
        config_files = [file for file in glob.glob(iws_defs.lister_directory + "*.json")]


        # Create the ui

        quit_button = QPushButton("Quit")
        quit_button.clicked.connect(self.quit)

        self.statusbar = QStatusBar(self)
        self.statusbar.setObjectName("statusbar")
        self.statusbar.showMessage('')
        self.setStatusBar(self.statusbar)

#        self.source_box = QtWidgets.QLabel('')
        self.statusbar.reformat()
#        self.statusbar.addPermanentWidget(self.source_box)
        self.statusbar.addPermanentWidget(quit_button)
        self.setStatusBar(self.statusbar)

        frame = QFrame()

        left_frame = self.create_left_frame()
        middle_frame = self.create_middle_frame()
        right_frame = self.create_right_frame(config_files)

        layout = QHBoxLayout()
#        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(left_frame)
        layout.addWidget(middle_frame)
        layout.addWidget(right_frame)

        frame.setLayout(layout)

        self.setCentralWidget(frame)

        finish = QAction("Quit", self)
        finish.triggered.connect(self.closeEvent)

        quit = QAction("Quit", self)
        quit.triggered.connect(self.closeEvent)


    def create_left_frame(self):
        font = QFont()
        font.setPointSize(9)
        #font.setBold(True)
        font.setWeight(75)

        box = QGroupBox("Data Source")
        self.src_list_widget = QListWidget()
        i = 0
        for src in self.sources:
            self.src_list_widget.insertItem(i, src)
            i += 1
        self.src_list_widget.currentItemChanged.connect(self.data_source_changed)
        self.src_list_widget.setCurrentRow(0)
        self.source = self.src_list_widget.item(0).text()

        browse_button = QPushButton("Browse")
        browse_button.clicked.connect(self.browse_button_clicked)

        self.delete_button = QPushButton("Delete")
        self.delete_button.setEnabled(True)
        self.delete_button.clicked.connect(self.delete_button_clicked)

        self.catalog_button = QPushButton("Catalog")
        self.catalog_button.clicked.connect(self.do_catalog)

        list_buttons_frame = QFrame()
        list_buttons_layout = QHBoxLayout()
        list_buttons_layout.setContentsMargins(0, 0, 0, 0)
        list_buttons_layout.addWidget(self.delete_button)
        list_buttons_layout.addWidget(browse_button)
        list_buttons_layout.addWidget(self.catalog_button)
        list_buttons_frame.setLayout(list_buttons_layout)

        layout = QVBoxLayout()
        layout.addWidget(self.src_list_widget)
#        layout.addStretch(1)
        layout.addWidget(list_buttons_frame)

        box.setLayout(layout)

        return box


    def create_middle_frame(self):
        font = QFont()
        font.setPointSize(9)
        #font.setBold(True)
        font.setWeight(75)

        box = QGroupBox("Catalog")

        layout = QVBoxLayout()
#        layout.setContentsMargins(0, 0, 0, 0)

        catalog_box = QGroupBox("Catalog")
        self.catalog_list = QListWidget()

        self.regenerate_catalog_list()
        self.catalog_list.currentItemChanged.connect(self.catalog_selection_changed)

        time_frame1 = QFrame()
        time_frame1_layout = QVBoxLayout()
        time_frame1_layout.setContentsMargins(0, 0, 0, 0)

        self.start_label = QLabel('Start datetime:')
        self.finish_label = QLabel('End datetime:')

        self.catalog_file_label = QLabel('')
        time_frame1_layout.addWidget(self.catalog_file_label)
        time_frame1_layout.addWidget(self.start_label)
        time_frame1_layout.addWidget(self.finish_label)

        self.update_catalog_labels()
        time_frame1.setLayout(time_frame1_layout)

        layout.addWidget(self.catalog_list)
        layout.addWidget(time_frame1)

        box.setLayout(layout)
        return box



    def create_right_frame(self, config_file):
        font = QFont()
        font.setPointSize(9)
        #font.setBold(True)
        font.setWeight(75)

        config_box = QGroupBox("Playback")

        config_box_layout = QVBoxLayout()
        self.configList = QListWidget()

        i = 0
        for file in sorted(config_file):
            #self.configList.insertItem(i, file)
            self.configList.insertItem(i, os.path.basename(file))
            i += 1

        self.configList.setCurrentRow(0)
        self.configuration = iws_defs.lister_directory + self.configList.item(0).text()
#        print(f"self.configuration = {self.configuration}")
        self.configList.currentItemChanged.connect(self.config_changed)

        buttons_frame = QFrame()
        buttons_frame_layout = QHBoxLayout()
        buttons_frame_layout.setContentsMargins(0, 0, 0, 0)

        refresh_button = QPushButton('Refresh')
        refresh_button.clicked.connect(self.refresh)
        button = QPushButton('Open Playback')
        button.clicked.connect(self.open_playback)
        buttons_frame_layout.addWidget(refresh_button)
        buttons_frame_layout.addWidget(button)
        buttons_frame.setLayout(buttons_frame_layout)

        config_box_layout.addWidget(self.configList)
        config_box_layout.addWidget(buttons_frame)

        config_box.setLayout(config_box_layout)
        return config_box


    def update_catalog_labels(self):
        with open(self.current_catalog_file, 'r') as f:
            line = f.read()
        items = line.split(',')
        self.catalog_file_label.setText(f"File: {items[0]}")
        self.start_datetime = items[1]
        self.start_label.setText(f"Start datetime: {items[1]}")
        self.end_datetime = items[2]
        self.finish_label.setText(f"End   datetime: {items[2]}")
        self.current_catalog_file = items[0]


    def catalog_selection_changed(self, curr, prev):
        if curr == None:
            return
        self.current_catalog_file = iws_defs.catalog_directory + curr.text()
        self.update_catalog_labels()
        pass


    def regenerate_catalog_list(self):
        self.catalog_list.clear()
        catalog_files = [file for file in glob.glob(iws_defs.catalog_directory + "*.txt")]
#        print(f"catalog files in {iws_defs.catalog_directory} = {catalog_files}")
        i = 0
        for c in catalog_files:
###            self.catalog_list.insertItem(i, c)
            self.catalog_list.insertItem(i, os.path.basename(c))
            i += 1
        self.catalog_list.setCurrentRow(0)
        self.current_catalog_file = iws_defs.catalog_directory + self.catalog_list.item(0).text()

# # extract_timing.sh csv_data_file catalog_directory
# subprocess.Popen(f"tail -f /media/pc_export/ex1.csv >> {self.source}&")
    def do_catalog(self):
#        result = subprocess.Popen(f"bash ./extract_timing.sh /Users/mcrw/Desktop/foo.csv {iws_defs.catalog_directory}")
        try:
##            process = subprocess.Popen(f"./extract_timing.sh /Users/mcrw/Desktop/foo.csv {iws_defs.catalog_directory} {self.protocol}", shell=True)
            # but self.source is the pc file...
####            process = subprocess.Popen(f"bash ./extract_timing.sh {self.source} {iws_defs.catalog_directory} {self.protocol}", shell=True)
            print(f"Running catalog on {self.source}")
            process = subprocess.Popen(f"bash ./extract_timing.sh {self.source} {iws_defs.catalog_directory}", shell=True)

            result = process.wait()
            if result == 0:
                self.statusbar.showMessage(f"Added {self.source} to the catalog ({result})")
                self.regenerate_catalog_list()
            else:
                self.statusbar.showMessage(f"Failed to add {self.source} to the catalog ({result})")
        except Exception as e:
            self.statusbar.showMessage(f"Failed to add {self.source} to the catalog ({result})")
#            print(f"Failed to add {self.source} to the catalog: {e}")


    def delete_button_clicked(self):
        filename = self.src_list_widget.currentItem().text()
        row = self.src_list_widget.currentRow()
        self.src_list_widget.takeItem(row)
        row = self.src_list_widget.currentRow()
        self.source = self.src_list_widget.item(row).text()
        self.statusbar.showMessage(f"Deleted {filename} from the Data Source list")
        pass


    def browse_button_clicked(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'csv file (*.csv)'
        path = os.environ['HOME']
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", path, suffixes, options=options)
        if filename:
            nitems = self.src_list_widget.count()
            # Check the file is not already in the list
            if not (filename in self.sources):
                self.src_list_widget.insertItem(nitems, filename)
                self.config.set('SOURCES', str(nitems), filename)
                self.src_list_widget.setCurrentItem(self.src_list_widget.item(nitems))
                self.source = filename
                self.sources = self.config['SOURCES'].values()
                self.srcs.append(filename)
            else:
                # Select it from the list
                nitems = self.sources.__len__()
                for k in range(nitems):
                    if filename == self.srcs[k]:
                        self.src_list_widget.setCurrentItem(self.src_list_widget.item(k))


    def refresh(self):
        self.configList.clear()
        config_files = [file for file in glob.glob(iws_defs.lister_directory + "*.json")]
        i = 0
        for file in sorted(config_files):
            #self.configList.insertItem(i, file)
            self.configList.insertItem(i, os.path.basename(file))
            i += 1

        self.configList.setCurrentRow(0)
        self.configuration = iws_defs.lister_directory + self.configList.item(0).text()
        pass


    def config_changed(self,curr, prev):
        if curr != None:
            self.configuration = iws_defs.lister_directory + curr.text()
        pass


    def data_source_changed(self, curr, prev):
        self.source = curr.text()


    def open_playback(self):
        config = self.configuration
#        print(f"python hk_playback.py {config} {self.current_catalog_file} {self.start_datetime} {self.end_datetime} &")
#        result = os.system(f"python hk_playback.py {config} {self.source} &")
###        result = os.system(f"python hk_playback.py {config} {self.current_catalog_file} {self.start_datetime} {self.end_datetime} &")
###        print(f"open playback {result}")
        pid = subprocess.Popen(['python', './hk_playback.py', f"{config}", f"{self.current_catalog_file}", f"{self.start_datetime}", f"{self.end_datetime}"])
#        print(f"open playback {result}")




    def closeEvent(self, event):
        with open(iws_defs.config_directory + 'playback.ini', 'w') as cfile:
            self.config.write(cfile)


    def quit(self):
        quitButtonReply = QtWidgets.QMessageBox.question(self,
                                                         'PyQt5 message', 'Are you sure you want to quit the program?',
                                                         QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No, QtWidgets.QMessageBox.No)
        if quitButtonReply == QtWidgets.QMessageBox.Yes:
            self.close()
            exit(1)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    #app.setStyleSheet("QGroupBox {background-color:pink;}")
    # Stolen from Themis' code...
    branch = os.popen('git rev-parse --abbrev-ref HEAD').read()[:-1]
    version = os.popen("git describe").read()[:-1]
    window = MockIWSCCWindow("HK CC (%s) (%s)" % (branch, version))
    window.show()

    sys.exit(app.exec_())
