import sys
import os
import re
import glob
import time
import subprocess

import configparser
import json

from PyQt5 import QtCore
from PyQt5.QtWidgets import QApplication, QWidget, QMainWindow, QScrollArea, QLabel, QGridLayout, QPushButton, QVBoxLayout, QFrame, QFileDialog, QInputDialog, QSpinBox, QComboBox #, QDialog, QDialogButtonBox
from PyQt5.QtWidgets import QMessageBox, QAction, QHBoxLayout, QTabWidget, QGroupBox, QRadioButton, QListWidget, QDateTimeEdit, QPlainTextEdit, QCheckBox
from PyQt5.QtCore    import Qt, QRect, QDate, QDateTime, QFileInfo
from PyQt5.QtGui     import QPalette, QColor, QFont

import iws_defs
import HkMonitorWindow

                    
#class MockIWSCCWindow(QWidget):
class MockIWSCCWindow(QMainWindow):
    def __init__(self, title, *args, **kwargs):
        super(MockIWSCCWindow, self).__init__(*args, **kwargs)

        self.scroll = QScrollArea()             # Scroll Area which contains the widgets, set as the centralWidget
        self.widget = QWidget()                 # Widget that contains the collection of Vertical Box
        # Set time format to UTC
        os.environ["TZ"] = "UTC"

        self.setWindowTitle(title)

        self.name = 'MainWindow'

        self.mode = ''
        self.connection = 'NoConn'
#        self.connected = False
        self.connected = True # TEMP
        self.export_mounted = False
        self.tail_process = 0 # or maybe None

        self.windows = []
        self.wdict = {}

        self.win = 0

        self.subWindow = None

        # Time range stuff
        self.anchor = ''
        self.start_datetime = QDateTime.currentDateTime()
        self.offset = ''
        self.end_datetime = QDateTime.currentDateTime()


        #self.configuration = 'packet_defaults.json'
        self.configuration = ''
        self.source = ''

        #self.config = configparser.ConfigParser(allow_no_value=True)
        self.config = configparser.ConfigParser()
        self.config.read(iws_defs.config_directory + 'launcher.ini')
        self.configs = self.config['CONFIGS'].values()
        self.sources = self.config['SOURCES'].values()

        # Take a stupid copy of the sources configuration because pyqt5 is stupid
        self.srcs = []
        for s in self.sources:
            self.srcs.append(s)

        # Get a list of configuration files
        config_files = [file for file in glob.glob(iws_defs.defaults_directory + "*.json")]

        # Create the ui

        lists_box_frame = self.create_lists_box_frame(config_files)
        protocol_frame = self.create_protocol_frame()
#        mode_frame = self.create_mode_frame()
        timespan_frame = self.create_timespan_timepoint_frame()
#        launch_buttons_frame = self.create_launch_buttons_frame()

        log_frame = self.create_log_frame()

        launch_box = QGroupBox()
        launch_box_layout = QVBoxLayout()

        launch_box_layout.addWidget(lists_box_frame)
        launch_box_layout.addWidget(protocol_frame)
#        launch_box_layout.addWidget(launch_buttons_frame)
#        launch_box_layout.addWidget(mode_frame)
        launch_box_layout.addWidget(timespan_frame)

        launch_box.setLayout(launch_box_layout)

        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(launch_box)

        layout.addWidget(log_frame)


        #self.setLayout(layout)
        self.widget.setLayout(layout)
        finish = QAction("Quit", self)
        finish.triggered.connect(self.closeEvent)

        quit = QAction("Quit", self)
        quit.triggered.connect(self.closeEvent)

        self.left_radio.setChecked(True)
        self.scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll.setWidgetResizable(True)
        self.scroll.setWidget(self.widget)
        self.setCentralWidget(self.scroll)


    def create_protocol_frame(self):
        frame = QFrame()
        frame_layout = QHBoxLayout()
        frame_layout.setContentsMargins(0, 0, 0, 0)

        protocol_frame = QGroupBox('Connection protocol')
        protocol_layout = QVBoxLayout()

        protocol_top_frame = QFrame()
        protocol_top_layout = QHBoxLayout()

        protocol_radio_frame = QFrame()
        protocol_radio_layout = QVBoxLayout()
        protocol_ccs_radio = QRadioButton("CCS")
        protocol_edds_radio = QRadioButton("EDDS")
        protocol_ccs_radio.clicked.connect(self.protocol_radio_toggle)
        protocol_edds_radio.clicked.connect(self.protocol_radio_toggle)
        protocol_ccs_radio.setEnabled(True)
        protocol_ccs_radio.setChecked(True)
        protocol_ccs_radio.setObjectName('ccs_radio')
        protocol_edds_radio.setEnabled(False)
        protocol_edds_radio.setObjectName('edds_radio')
        protocol_radio_layout.addWidget(protocol_ccs_radio)
        protocol_radio_layout.addWidget(protocol_edds_radio)
        protocol_radio_frame.setLayout(protocol_radio_layout)

        self.protocol_connect_button = QPushButton('Connect')
        self.protocol_connect_button.clicked.connect(self.connect_to_CCS)
        protocol_top_layout.addWidget(protocol_radio_frame)
        protocol_top_layout.addWidget(self.protocol_connect_button)
        protocol_top_layout.addStretch(1)
        protocol_top_frame.setLayout(protocol_top_layout)

        self.protocol_connected_label = QLabel()
        self.protocol_connected_label.setText('Status: Disconnected')

        self.protocol_ip_label = QLabel()
        self.protocol_ip_label.setText('Server IP: Unknown')

        protocol_layout.addWidget(protocol_top_frame)
        protocol_layout.addWidget(self.protocol_connected_label)
        protocol_layout.addWidget(self.protocol_ip_label)

        protocol_frame.setLayout(protocol_layout)

        launch_buttons_frame = self.create_launch_buttons_frame()

        request_frame = self.create_request_frame()

        info_frame = self.create_info_frame()

        frame_layout.addWidget(protocol_frame)
#        frame_layout.addStretch(1)
        frame_layout.addWidget(launch_buttons_frame)
#        frame_layout.addStretch(1)
        frame_layout.addWidget(request_frame)
#        frame_layout.addStretch(1)
        frame_layout.addWidget(info_frame)
###        frame_layout.addStretch(1)

        button_frame = QFrame()
        button_frame_layout = QVBoxLayout()
        define_timespan_button = QPushButton('Define Timespan')
        define_timespan_button.clicked.connect(self.define_timespan_clicked)
        define_dumpspan_button = QPushButton('Define from dump file')
        button_frame_layout.addWidget(define_timespan_button)
        button_frame_layout.addWidget(define_dumpspan_button)
        button_frame_layout.addStretch(1)
        button_frame.setLayout(button_frame_layout)

        frame_layout.addWidget(button_frame)

        frame.setLayout(frame_layout)

        #return protocol_frame
        return frame


    def create_info_frame(self):
#        frame = QFrame()
        frame = QGroupBox('Information')
        layout = QVBoxLayout()

        l = QLabel('Timespan specified:')
        self.from_label = QLabel('From:')
        self.to_label = QLabel('To:')
        self.time_label = QLabel('0 sec')
        self.tdelta_label = QLabel('TDelta:')
        
        layout.addWidget(l)
        layout.addWidget(self.from_label)
        layout.addWidget(self.to_label)
        layout.addWidget(self.time_label)
        layout.addWidget(self.tdelta_label)

        frame.setLayout(layout)
        return frame


    def create_lists_box_frame(self, config_files):
        list_box = QFrame()
        list_box.setMinimumSize(QtCore.QSize(100, 200))
        list_box_layout = QHBoxLayout()
        list_box_layout.setContentsMargins(0, 0, 0, 0)

        
        config_box = QGroupBox("Configuration")
        self.configList = QListWidget()

        i = 0
        for file in sorted(config_files):
            self.configList.insertItem(i, file)
            i += 1

        self.configList.setCurrentRow(0)
        self.configuration = self.configList.item(0).text()
        self.configList.currentItemChanged.connect(self.config_changed)

        self.update_button = QPushButton('Update')
        self.update_button.clicked.connect(self.update_config)

        update_frame = QFrame()
        update_layout = QHBoxLayout()
        update_layout.setContentsMargins(0, 0, 0, 0)
        update_layout.addWidget(self.update_button)
        update_layout.addStretch(1)
        update_frame.setLayout(update_layout)

        config_layout = QVBoxLayout()
        config_layout.addWidget(self.configList)
        config_layout.addWidget(update_frame)
        config_box.setLayout(config_layout)

        data_box = QGroupBox("Data Source")
        self.listWidget = QListWidget()
        i = 0
        for src in self.sources:
            self.listWidget.insertItem(i, src)
            i += 1
        self.listWidget.currentItemChanged.connect(self.data_source_changed)

        self.browse_button = QPushButton("Browse")
        self.browse_button.clicked.connect(self.browse_button_clicked)

        self.delete_button = QPushButton("Delete")
        self.delete_button.setEnabled(False)
        self.delete_button.clicked.connect(self.delete_button_clicked)

        
        list_buttons_frame = QFrame()
        list_buttons_layout = QHBoxLayout()
        list_buttons_layout.setContentsMargins(0, 0, 0, 0)
        list_buttons_layout.addWidget(self.delete_button)
        list_buttons_layout.addWidget(self.browse_button)
        list_buttons_frame.setLayout(list_buttons_layout)

        data_layout = QVBoxLayout()
        data_layout.addWidget(self.listWidget)
        data_layout.addWidget(list_buttons_frame)
        data_layout.addStretch(1)

        data_box.setLayout(data_layout)

        list_box_layout.addWidget(config_box)
        list_box_layout.addWidget(data_box)
        list_box.setLayout(list_box_layout)
        return list_box


    def create_mode_frame(self):
        mode_frame = QFrame()
        mode_frame.setFrameStyle(QFrame.StyledPanel)
        #mode_frame.setStyleSheet("background-color:lightgreen;")
        mode_frame_layout = QVBoxLayout()
        left_frame = self.create_left_timerange_frame()
        middle_frame = self.create_middle_timerange_frame()
        right_frame = self.create_right_timerange_frame()
        mode_frame_layout.addWidget(left_frame)
        mode_frame_layout.addWidget(middle_frame)
        mode_frame_layout.addWidget(right_frame)
        mode_frame.setLayout(mode_frame_layout)
        return mode_frame
    

    def create_launch_buttons_frame(self):
        launch_button_frame = QFrame()
        launch_button_frame_layout = QHBoxLayout()
        launch_button_frame_layout.setContentsMargins(0, 0, 0, 0)
        close_button_frame = QFrame()
        close_button_frame_layout = QHBoxLayout()
        close_button_frame_layout.setContentsMargins(0, 0, 0, 0)

        self.launch_button = QPushButton("Launch HKTM Monitor")
        self.close_button = QPushButton("Close HKTM Monitor")
        self.launch_button.setEnabled(False)
        self.close_button.setEnabled(False)
        self.launch_button.clicked.connect(self.open_subwindow)
        self.close_button.clicked.connect(self.close_subwindow)

        launch_button_frame_layout.addStretch(1)
        launch_button_frame_layout.addWidget(self.launch_button)
        launch_button_frame_layout.addStretch(1)
        launch_button_frame.setLayout(launch_button_frame_layout)

        close_button_frame_layout.addStretch(1)
        close_button_frame_layout.addWidget(self.close_button)
        close_button_frame_layout.addStretch(1)
        close_button_frame.setLayout(close_button_frame_layout)

#        buttons_frame = QFrame()
        buttons_frame = QGroupBox('Launch')
#        buttons_frame.setFrameStyle(QFrame.StyledPanel)
        buttons_frame_layout = QVBoxLayout()
        buttons_frame_layout.addWidget(launch_button_frame)
        buttons_frame_layout.addWidget(close_button_frame)
        buttons_frame.setLayout(buttons_frame_layout)
        return buttons_frame


    def create_radios(self):
        f = QFrame()
        return f

    def create_timespan_timepoint_frame(self):
        frame = QGroupBox('Define Timespan with anchor at timepoint:')
        layout = QVBoxLayout()

        left_frame = self.create_left_timerange_frame()
        middle_frame = self.create_middle_timerange_frame()
        right_frame = self.create_right_timerange_frame()
        
        layout.addWidget(left_frame)
        layout.addWidget(middle_frame)
        layout.addWidget(right_frame)

        frame.setLayout(layout)
        return frame


    def create_left_timerange_frame(self):
        frame = QFrame()
        layout = QHBoxLayout()

        self.left_radio = QRadioButton('Left Edge')
        self.left_radio.setObjectName('left')
        self.left_radio.toggled.connect(self.timepoint_select)
#        self.left_radio.setChecked(True)
        self.left_timerange_n0    = QPushButton('N')
        self.left_timerange_n1    = QPushButton('N')
        self.left_start_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        self.left_start_time_box.setDisplayFormat("yyyy/MM/ddTHH:mm:ss")
        self.left_start_time_box.dateTimeChanged.connect(self.start_datetime_changed)
        l0 = QLabel('and')
        l1 = QLabel('after, or until')
        self.left_timerange_seconds_after = QSpinBox()
        self.left_timerange_seconds_after.valueChanged.connect(self.offset_changed)
        self.left_timerange_time_span = QComboBox()
        self.left_timerange_time_span.addItem('Sec')
        self.left_end_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        self.left_end_time_box.setDisplayFormat("yyyy/MM/ddTHH:mm:ss")
        self.left_end_time_box.dateTimeChanged.connect(self.end_datetime_changed)
        """
        radio = QRadioButton('Left Edge')
        self.left_timerange_n0    = QPushButton('N')
        self.left_timerange_n1    = QPushButton('N')
        self.left_start_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        self.left_start_time_box.setDisplayFormat("yyyy/MM/ddTHH:mm:ss")
        l0 = QLabel('and')
        l1 = QLabel('after, or until')
        self.left_timerange_seconds_after = QSpinBox()
        self.left_timerange_time_span = QComboBox()
        self.left_timerange_time_span.addItem('Sec')
        self.left_end_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        self.left_end_time_box.setDisplayFormat("yyyy/MM/ddTHH:mm:ss")
        """

        layout.addWidget(self.left_radio)
        layout.addWidget(self.left_timerange_n0)
        layout.addWidget(self.left_start_time_box)
        layout.addWidget(l0)
        layout.addWidget(self.left_timerange_seconds_after)
        layout.addWidget(self.left_timerange_time_span)
        layout.addWidget(l1)
        layout.addWidget(self.left_end_time_box)
        layout.addWidget(self.left_timerange_n1)
        frame.setLayout(layout)
        return frame


    def create_middle_timerange_frame(self):
        frame = QFrame()
        layout = QHBoxLayout()

        self.mid_radio = QRadioButton('Middle Point')
        self.mid_radio.setObjectName('mid')
        self.mid_radio.toggled.connect(self.timepoint_select)
        n0    = QPushButton('N')
        n1    = QPushButton('N')
        start_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        start_time_box.setDisplayFormat("yyyy/MM/ddTHH:mm:ss")
        start_time_box.dateTimeChanged.connect(self.start_datetime_changed)
        l0 = QLabel('and')
        l1 = QLabel('before, and')
        timerange_seconds_before = QSpinBox()
        timerange_seconds_before.valueChanged.connect(self.offset_changed)
        timerange_time_span = QComboBox()
        timerange_time_span.addItem('Sec')
        end_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        end_time_box.setDisplayFormat("yyyy/MM/ddTHH:mm:ss")
        end_time_box.dateTimeChanged.connect(self.end_datetime_changed)

        layout.addWidget(self.mid_radio)
        layout.addWidget(n0)
        layout.addWidget(start_time_box)
        layout.addWidget(l0)
        layout.addWidget(timerange_seconds_before)
        layout.addWidget(timerange_time_span)
        layout.addWidget(l1)
        layout.addWidget(end_time_box)
        layout.addWidget(n1)
        frame.setLayout(layout)
        return frame


    def create_right_timerange_frame(self):
        frame = QFrame()
        layout = QHBoxLayout()

        self.right_radio = QRadioButton('Right Edge')
        self.right_radio.setObjectName('right')
        self.right_radio.toggled.connect(self.timepoint_select)
        n0    = QPushButton('N')
        n1    = QPushButton('N')
        start_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        start_time_box.setDisplayFormat("yyyy/MM/ddTHH:mm:ss")
        start_time_box.dateTimeChanged.connect(self.start_datetime_changed)
        l0 = QLabel('and')
        l1 = QLabel('before, or from')
        timerange_seconds_before = QSpinBox()
        timerange_seconds_before.valueChanged.connect(self.offset_changed)
        timerange_time_span = QComboBox()
        timerange_time_span.addItem('Sec')
        end_time_box = QDateTimeEdit(QDateTime.currentDateTime())
        end_time_box.setDisplayFormat("yyyy/MM/ddTHH:mm:ss")
        end_time_box.dateTimeChanged.connect(self.end_datetime_changed)

        layout.addWidget(self.right_radio)
        layout.addWidget(n0)
        layout.addWidget(start_time_box)
        layout.addWidget(l0)
        layout.addWidget(timerange_seconds_before)
        layout.addWidget(timerange_time_span)
        layout.addWidget(l1)
        layout.addWidget(end_time_box)
        layout.addWidget(n1)
        frame.setLayout(layout)
        return frame


    def toggle_buttons(self, name):
#        name = button.objectName()
        if name == 'left':
            self.mid_radio.setChecked(False)
            self.right_radio.setChecked(False)
        elif name == 'mid':
            self.left_radio.setChecked(False)
            self.right_radio.setChecked(False)
        else:
            self.left_radio.setChecked(False)
            self.mid_radio.setChecked(False)


    def timepoint_select(self):
        button = self.sender()
        if button.isChecked():
            self.anchor = button.text()
            name = button.objectName()
#            print(f"radio clicked set: {button.objectName()}")
#            self.log('INFO', f"radio clicked set: {button.objectName()}")
            self.toggle_buttons(name)
        else:
#            print(f"radio clicked unset: {button.objectName()}")
#            self.log('INFO', f"radio clicked unset: {button.objectName()}")
            pass


    def create_request_frame(self):
        frame = QGroupBox('Request')
#        frame = QFrame()
#        frame.setFrameStyle(QFrame.StyledPanel)
        layout = QVBoxLayout()

        self.real_time_mode_button = QPushButton('Real Time Mode')
        self.request_hk_button = QPushButton('Request HK/TM Metadata')
        self.stop_acquisition_button = QPushButton('Stop HK/TM Acquisition')
        
        self.real_time_mode_button.setEnabled(False)
        self.request_hk_button.setEnabled(False)
        self.stop_acquisition_button.setEnabled(False)
        
        self.real_time_mode_button.clicked.connect(self.realtime_mode_button_pressed)
        self.request_hk_button.clicked.connect(self.request_hk_button_pressed)
        self.stop_acquisition_button.clicked.connect(self.stop_acquisition_button_pressed)

        layout.addWidget(self.real_time_mode_button)
        layout.addWidget(self.request_hk_button)
        layout.addWidget(self.stop_acquisition_button)
        frame.setLayout(layout)
        return frame


    def create_log_frame(self):
        frame = QGroupBox('Log')
        frame.setMinimumSize(QtCore.QSize(100, 300))
        layout = QVBoxLayout()

        font = QFont("Consolas", 8)
#        font.setPointSize(8)
#        self.setFont(font)

        self.log_text_box = QPlainTextEdit()
        self.log_text_box.setReadOnly(True)
        self.log_text_box.setFont(font)

        button_frame = QFrame()
        button_frame_layout = QHBoxLayout()

        clear_button = QCheckBox('Clear log display area before every request')
        write_button = QCheckBox('Write requests and results into logfile')

        button_frame_layout.addWidget(clear_button)
        button_frame_layout.addWidget(write_button)
        button_frame.setLayout(button_frame_layout)

        layout.addWidget(self.log_text_box)
#        layout.addStretch(1)
        layout.addWidget(button_frame)
        frame.setLayout(layout)
        return frame


    # Needs error checking...
    # Uncomment either mount command line - !!!password needs to be typed into the xterm at the moment!!!
    def mount_cifs(self):
#        reply = os.popen("sudo mount.cifs //WIN10THING/share /media/pc_export -o user=mcrw")
#        reply = os.popen("sudo mount -t cifs //WIN10THING/share /media/pc_export -o user=mcrw")
#        print(f"mount.cifs reply = {reply}")
        pass


    def connect_to_CCS(self):
        if self.export_mounted == False:
            self.mount_cifs()
            self.export_mounted = True

        button = self.sender()
        if button.text() == 'Connect':
        #if self.sender().text() == 'Connect':
            button.setEnabled(False)
            ip_addr, ok = QInputDialog.getText(self, 'Input Dialog', 'Enter CCS server ip address')

            if ok:
                # TODO: check ip_addr string is correctly formed
                print(f"ip address = {ip_addr}")
                ok = self.check_CCS_connectivity(ip_addr)
                print(f"Check returned {ok}")
                if ok:
                    self.protocol_ip_label.setText('Server IP: %s' % ip_addr)
                    self.protocol_connected_label.setText('Status: Connected')
                    self.protocol_connect_button.setText('Disconnect')
                    self.connected = True
                else:
                    self.protocol_connected_label.setText('Status: Disconnected')
                    self.protocol_ip_label.setText('Server IP: Not found')
        else: # button text is Disonnect
            self.disconnect_from_CCS()
            pass
        
        button.setEnabled(True)


    def disconnect_from_CCS(self):
        # Pretty much a null action
        button = self.sender()
        button.setText('Connect')
        self.protocol_connected_label.setText('Status: Disconnected')
        self.protocol_ip_label.setText('Server IP: Unknown')
        self.connected = False
        pass


    # Uncomment to get true functionality (commented out for testing)
    def check_CCS_connectivity(self, server_ip_addr):
        ret = True
#        self.protocol_connected_label.setText('Status: Checking...')
#        QApplication.processEvents()
#        ping_reply = os.popen(f"ping -qn -c 1 {server_ip_addr}").readlines()
#        for ping_string in ping_reply:
#            print(f"ping_string = {ping_string}")
#            if re.search("100%", ping_string):
#                print("BAD")
#                ret = False
#                break
        #ret = True # for testing
        return ret
        pass

    
    def realtime_mode_button_pressed(self):
        self.real_time_mode_button.setEnabled(False)
        self.request_hk_button.setEnabled(True)
        self.stop_acquisition_button.setEnabled(True)
        self.subWindow.request_hktm_metadata(self.connection, '-1', '-1')
        #self.subWindow.realtime()


    def request_hk_button_pressed(self):
#        dt = self.start_time_box.dateTime()
#        #fmt = self.start_time_box.displayFormat()
#        fmt = "yyyy-MM-ddTHH:mm:ss"
#        dt_from_string = dt.toString(fmt)
#        dt = self.end_time_box.dateTime()
#        dt_to_string = dt.toString(fmt)
#        print(f"mock request_hktm_metadata {self.connection}, {dt_from_string}, {dt_to_string}")
#        self.subWindow.request_hktm_metadata(self.connection, dt_from_string, dt_to_string)
        pass


    def stop_acquisition_button_pressed(self):
        self.real_time_mode_button.setEnabled(True)
        self.request_hk_button.setEnabled(True)
        self.subWindow.stop_acquisition()


    def protocol_radio_toggle(self):
        button = self.sender()
        if button.objectName() == "ccs_radio":
            self.connection = "CCS"
        else :
            self.connection = "EDDS"
        print(f"Connection is {self.connection}")


    def mode_selection_toggle(self):
        button = self.sender()
        if button.objectName() == "realtime_button":
            self.mode = "realtime"
            self.start_time_box.setEnabled(False)
            self.end_time_box.setEnabled(False)
        else :
            self.mode = "replay"
            self.start_time_box.setEnabled(True)
            self.end_time_box.setEnabled(True)
        pass


    def update_config(self):
        self.configList.clear()
        config_files = [file for file in glob.glob(iws_defs.defaults_directory + "*.json")]
        i = 0
        for file in sorted(config_files):
            self.configList.insertItem(i, file)
            i += 1
        self.configList.setCurrentRow(0)
        self.configuration = self.configList.item(0).text()
        pass


    def delete_button_clicked(self):
        pass


    def browse_button_clicked(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'csv file (*.csv)'
        path = os.environ['HOME']
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", path, suffixes, options=options)
        if filename:
            nitems = self.listWidget.count()
            # Check the file is not already in the list
            if not (filename in self.sources):
                self.listWidget.insertItem(nitems, filename)
                self.config.set('SOURCES', str(nitems), filename)
                self.listWidget.setCurrentItem(self.listWidget.item(nitems))
                self.source = filename
                self.sources = self.config['SOURCES'].values()
                self.srcs.append(filename)
            else:
                # Select it from the list
                nitems = self.sources.__len__()
                for k in range(nitems):
                    if filename == self.srcs[k]:
                        self.listWidget.setCurrentItem(self.listWidget.item(k))


    def config_changed(self,curr, prev):
        if curr != None:
            self.configuration = curr.text()
        pass


    def data_source_changed(self, curr, prev):
        self.source = curr.text()
        self.delete_button.setEnabled(True)
        self.launch_button.setEnabled(True)


    def open_subwindow(self):
        if self.subWindow == None:
# TEMP - comment back in or get user to do this in an xterm (simpler and more likely to work)
#            os.system(f"cat /media/pc_export/ex1.csv > {self.source}")
#            #os.system(f"tail -f /media/pc_export/ex1.csv >> {self.source}&")
#            self.tail_process = subprocess.Popen(f"tail -f /media/pc_export/ex1.csv >> {self.source}&")

            if self.connected == True:
                self.close_button.setEnabled(True)
                self.launch_button.setEnabled(False)
                self.real_time_mode_button.setEnabled(True)
                self.request_hk_button.setEnabled(True)
                #s = HkMonitorWindow.HkMonitorWindow(config=os.path.basename(self.configuration), source=self.source, mode=realtime, start='', end='')
                s = HkMonitorWindow.HkMonitorWindow(config=os.path.basename(self.configuration), source=self.source)
                print(f"HkMonitorWindow.HkMonitorWindow(config={os.path.basename(self.configuration)}, source={self.source})")
                #s = HkMonitorWindow.HkMonitorWindow(os.path.basename(self.configuration), self.source)
                self.subWindow = s
                s.show()
            else:
                print("Connect to a server first...")


    def close_subwindow(self):
        if self.subWindow != None:
            ok = self.subWindow.closeWindow()
            self.subWindow = None
            #print("HkMonitor window close ok: " + str(ok))
            self.close_button.setEnabled(False)
            self.launch_button.setEnabled(True)
            self.real_time_mode_button.setEnabled(False)
            self.request_hk_button.setEnabled(False)
            self.stop_acquisition_button.setEnabled(False)


    def closeEvent(self, event):
        with open(iws_defs.config_directory + 'launcher.ini', 'w') as cfile:
            self.config.write(cfile)
            if self.export_mounted:
                # Do we really want to do this?
#                reply = os.popen("sudo umount -i -t cifs /media/pc_export")
                pass
#            if self.tail_process !=0: # or None
#                self.tail_process.kill()


    def offset_changed(self):
        spinner = self.sender()
        self.offset = spinner.value()


    def start_datetime_changed(self, new_datetime):
        self.start_datetime = new_datetime


    def end_datetime_changed(self, new_datetime):
#        print(f"new date time: {new_datetime}")
        self.end_datetime = new_datetime


    def define_timespan_clicked(self):
        enddatestring = self.end_datetime.toString("yyyyMMdd_hh:mm:ss")
        endms = self.end_datetime.toSecsSinceEpoch()
        self.to_label.setText(f"To: {enddatestring}")
        startdatestring = self.start_datetime.toString("yyyyMMdd_hh:mm:ss")
        startms = self.start_datetime.toSecsSinceEpoch()
        self.from_label.setText(f"From: {startdatestring}")
        duration = endms - startms
        self.time_label.setText(f"{duration} sec")
        self.log('INFO', 'Define Timespan push button pressed')
        self.log('INFO', f"Determining Timespan using anchor at timepoint: {self.anchor}")
        self.log('INFO', f"Timespan from : {startdatestring} ({startms} sec)")
        self.log('INFO', f"Timespan to   : {enddatestring} ({endms} sec)")
        self.log('INFO', f"Timespan duration : {duration} sec")

    def log(self, lvl, msg):
        datetime = QDateTime.currentDateTime().toString("yyyy-MM-dd hh:mm:ss.zzz")
        self.log_text_box.appendPlainText(f"{datetime} - {lvl} - {msg}")


if __name__ == '__main__':
    #Peter: pdb start here
    #import pdb; pdb.set_trace()
    app = QApplication(sys.argv)
    #app.setStyleSheet("QGroupBox {background-color:pink;}")
    # Stolen from Themis' code...
    branch = os.popen('git rev-parse --abbrev-ref HEAD').read()[:-1]
    version = os.popen("git describe").read()[:-1]
    window = MockIWSCCWindow("Mock IWS CC (%s) (%s)" % (branch, version))

    #   Scroll Area Properties
    #scroll = QScrollArea()
    #scroll.setFrameShape(frame)
    #scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
    #scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
    #scroll.setWidgetResizable(True)
    #scroll.setWidget(window)

    #scroll_layout = QVBoxLayout()
    #scroll_layout.addWidget(scroll)
    #scroll_layout.setContentsMargins(0, 0, 0, 0)
    #self.setLayout(scroll_layout) 
    #window.setLayout(scroll_layout) 
    window.show()

    sys.exit(app.exec_())
