from PyQt5 import QtWidgets, QtGui, QtCore
from PyQt5.QtCore import Qt, QFileInfo
from PyQt5.QtWidgets import QFileDialog, QMessageBox

import sys 

import json
import re

import iws_defs
import chooser_defs

from lib.Lists.Lists import ListItem, ChildListItem, ParentListItem, List, MasterList
#import ListItem
#import ChildListItem
#import ParentListItem
#import List
#import MasterList

XTOP = 0
YTOP = 0
WIDTH = 600
HEIGHT = 600

choices = ['system', 'packets']

system_names = ['CDPU', 'OBS-G', 'PMCU', 'ROE', 'RPSU']
system_tabs = {'CDPU'  : chooser_defs.cdpu_parameters,
               'OBS-G' : chooser_defs.obsg_parameters,
               'PMCU'  : chooser_defs.pmcu_parameters,
               'ROE'   : chooser_defs.roe_parameters,
               'RPSU'  : chooser_defs.rpsu_parameters}

packet_names = ['RT_FAST', 'RT_SLOW', 'RT_SUPERFAST', 'ST_SLOW', 'ST_SUPERFAST']
packet_tabs = {'RT_FAST'      : chooser_defs.rt_fast_parameters,
               'RT_SLOW'      : chooser_defs.rt_slow_parameters,
               'RT_SUPERFAST' : chooser_defs.rt_superfast_parameters,
               'ST_SLOW'      : chooser_defs.st_slow_parameters,
               'ST_SUPERFAST' : chooser_defs.st_superfast_parameters}

class chooserWindow(QtWidgets.QMainWindow):

    def __init__(self):
        super(chooserWindow, self).__init__()
        #print("__init__")
        #self.trees = []

        """
        self.system_trees = {'CDPU'  : [],
                             'OBS-G' : [],
                             'PMCU'  : [],
                             'ROE'   : [],
                             'RPSU'  : []}

        self.packet_trees = {'RT_FAST'      : [],
                             'RT_SLOW'      : [],
                             'RT_SUPERFAST' : [],
                             'ST_SLOW'      : [],
                             'ST_SUPERFAST' : []}
                             """

        self.system_trees = {'CDPU'  : None,
                             'OBS-G' : None,
                             'PMCU'  : None,
                             'ROE'   : None,
                             'RPSU'  : None}

        self.packet_trees = {'RT_FAST'      : None,
                             'RT_SLOW'      : None,
                             'RT_SUPERFAST' : None,
                             'ST_SLOW'      : None,
                             'ST_SUPERFAST' : None}

        self.parameter_trees = {'system'  : self.system_trees,
                                'packets' : self.packet_trees}

        self.type_choice = 'system'

        self.output_filename = ''
        self.output_dir = './'

        self.groups = []

        self.initUi()

        
    def initUi(self):
        #print("initUi")

        self.setObjectName("MainWindow")
        self.resize(WIDTH, HEIGHT)
        self.setWindowTitle("Parameter Lister")

        centralWidget = QtWidgets.QWidget(self)
        centralWidget.setObjectName("centralWidget")
        centralWidget.setAccessibleName("centralWidget")
        #centralWidget.setStyleSheet("background-color:purple;")

        mainMenu = self.menuBar()
        fileMenu = mainMenu.addMenu('File')
        quitAct = QtWidgets.QAction('Quit', self, checkable = False)
        fileMenu.addAction(quitAct)

        self.statusbar = QtWidgets.QStatusBar(self)
        self.statusbar.setObjectName("statusbar")
        self.statusbar.showMessage('')
        self.setStatusBar(self.statusbar)


        self.tabWidgetFrame = self.createTabWidgetFrame()
        #self.tabWidgetFrame.setMinimumSize(QtCore.QSize(210, 100))
        self.tabWidgetFrame.setMinimumSize(QtCore.QSize(iws_defs.TEXT_FIELD_MINIMUM_SIZE + 140, 100))

        control_frame = QtWidgets.QFrame()
        control_frame.setObjectName("control_frame")
        #control_frame.setStyleSheet("background-color:yellow;")
        control_frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)

        top_frame = QtWidgets.QFrame()
        #top_frame.setStyleSheet("background-color:lightgreen;")
        top_frame_layout = QtWidgets.QHBoxLayout()
        top_frame_layout.setContentsMargins(0, 0, 0, 0)

        list_frame = QtWidgets.QFrame()
        #list_frame.setStyleSheet("background-color:lightblue;")
        list_frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
        list_frame_layout = QtWidgets.QVBoxLayout()
        list_frame_layout.setContentsMargins(0, 0, 0, 0)

        group_list_box = QtWidgets.QGroupBox('Groups:')
        group_list_box_layout = QtWidgets.QVBoxLayout()
        #group_list_box_layout.setContentsMargins(0, 0, 0, 0)
        #self.group_list_view = QtWidgets.QListView()
        group_list_box.setMinimumSize(QtCore.QSize(iws_defs.TEXT_FIELD_MINIMUM_SIZE, 100))
        #self.group_list_view = QtWidgets.QListWidget()
        self.group_list_view = List("Groups")
        self.group_list_view.setAcceptDrops(False)

        list_box_button_frame = QtWidgets.QFrame()
        list_box_button_frame_layout = QtWidgets.QHBoxLayout()
        self.delete_group_button = QtWidgets.QPushButton('Delete')
        self.delete_group_button.clicked.connect(self.delete_group)
        list_box_button_frame_layout.addWidget(self.delete_group_button)
        list_box_button_frame_layout.addStretch(1)
        list_box_button_frame.setLayout(list_box_button_frame_layout)

        group_list_box_layout.addWidget(self.group_list_view)
        group_list_box.setLayout(group_list_box_layout)

        list_frame_layout.addWidget(group_list_box)
        #list_frame_layout.addWidget(self.delete_group_button)
        list_frame_layout.addWidget(list_box_button_frame)
        list_frame.setLayout(list_frame_layout)

        quitButton = QtWidgets.QPushButton("Quit")
        quitButton.clicked.connect(self.quitClicked)

        exportButton = QtWidgets.QPushButton("Export")
        exportButton.clicked.connect(self.exportClicked)

        cancelButton = QtWidgets.QPushButton("Cancel")
        cancelButton.clicked.connect(self.cancelClicked)

        hbox = QtWidgets.QHBoxLayout()
        #hbox.addStretch(1)
        #hbox.setContentsMargins(0, 0, 0, 0)
        hbox.addWidget(cancelButton)
        hbox.addWidget(exportButton)
        hbox.addStretch(1)
        hbox.addWidget(quitButton)

        control_frame.setLayout(hbox)

        main_layout = QtWidgets.QVBoxLayout()

        top_frame_layout.addWidget(self.tabWidgetFrame, stretch = 1)
        top_frame_layout.addWidget(list_frame)

        top_frame.setLayout(top_frame_layout)

        main_layout.addWidget(top_frame)
        main_layout.addWidget(control_frame, stretch = 0)

        centralWidget.setLayout(main_layout)

        self.setCentralWidget(centralWidget)



    def quitClicked(self):
        quitButtonReply = QtWidgets.QMessageBox.question(self,
                                                         'PyQt5 message', 'Are you sure you want to quit the program?',
                                                         QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No, QtWidgets.QMessageBox.No)
        if quitButtonReply == QtWidgets.QMessageBox.Yes:
            self.close()
            exit(1)


    def parameters_number_good(self, n_parameters):
        if n_parameters == 0:
            self.statusbar.showMessage('No parameters!')
            return False
        if n_parameters > 16:
            self.statusbar.showMessage('Too many parameters (' + str(n_parameters) + ') selected for group (12 maximum).')
            return False
        return True


    def delete_group(self):
        #print("delete group")
        rows = []
        ###self.group_list_view = List.List("Groups")
        selected_list_items = self.group_list_view.selectedItems()
        for list_item in selected_list_items:
            #print(f"list_item {list_item} at {self.group_list_view.row(list_item)}")
            row = self.group_list_view.row(list_item)
            rows.append(row)
            saved_item = self.groups[row]
            #print(f"saved_item = {saved_item}")
            main_tab_index = saved_item[0].main_tab
            sub_tab_index = saved_item[0].sub_tab
            #print(f"main, sub = {main_tab_index}, {sub_tab_index}")

            # select list in correct tab
            source = choices[main_tab_index]
            if main_tab_index == 0:
                sub_tab_name = system_names[sub_tab_index]
            else:
                sub_tab_name = packet_names[sub_tab_index]
            #print(f"sub_tab_name = {sub_tab_name}")

            # re-enable the items in the list to be deleted
            #print(f"{source} {self.parameter_trees[source]}")
            the_list = self.parameter_trees[source][sub_tab_name]
            #print(f"list: {the_list}")

            #print(f"groups[{row}] = {self.groups[row]}")
            for list_item in self.groups[row]:
                list_row = list_item.row
                #print(f"{the_list.item(list_row)}")
                the_list.item(list_row).enable()

            # delete list from self.groups
            #del self.groups[row]

        # delete from list view
        rows.sort(reverse=True)
        #print(f"rows = {rows}")
        for r in rows:
            self.group_list_view.takeItem(r)
            del self.groups[r]
            pass


    def groupClicked(self, b):
        current_tab_index = self.masterTabWidget.currentIndex()
        type = 'system' if current_tab_index == 0 else 'packets'
        names = system_names if type == 'system' else packet_names
        (current_sub_tab_name, current_sub_tab_index) = self.get_sub_tab_name()

        the_list = self.parameter_trees[type][current_sub_tab_name] # <List.List object>
        selected_items = the_list.selectedItems()
        n_parameters = len(selected_items)

        if not self.parameters_number_good(n_parameters):
            return

        first_item = the_list.get_item_at(0) # ListItem.ListItem
        (name, id, src, unit, row) = first_item.get_components()        

        if first_item.parent != '':
            name = first_item.parent

        #thing = QtWidgets.QListWidgetItem(first_item)
        thing = ListItem(name, id, src, unit, row)
        thing.main_tab = current_tab_index
        thing.sub_tab = current_sub_tab_index
        self.group_list_view.addItem(thing)

        list_array = []
        for item in the_list.selectedItems():
            item.disable()
            (name, id, src, unit, row) = item.get_components()
            new_item = ListItem(name, id, src, unit, row)
            new_item.main_tab = current_tab_index
            new_item.sub_tab = current_sub_tab_index
            list_array.append(new_item)

        self.groups.append(list_array)
        self.statusbar.showMessage(str(n_parameters) + ' parameters added to group.')


    def clearAllClicked(self, b):
        #print("clearAllClicked")
        current_tab = self.masterTabWidget.currentIndex()
        #current_tab_name = self.masterTabWidget.tabText(current_tab)
        #print(f"current_tab_name = {current_tab_name}")
        type = 'system' if current_tab == 0 else 'packets'
        names = system_names if type == 'system' else packet_names
        current_sub_tab_widget = self.masterTabWidget.currentWidget()
        #print(f"current_sub_tab = {current_sub_tab_widget}")
        current_sub_tab = current_sub_tab_widget.currentIndex()
        current_sub_tab_name = current_sub_tab_widget.tabText(current_sub_tab)
        #print(f"current_sub_tab_name = {current_sub_tab_name}")

        #print(f"self.parameter_trees = {self.parameter_trees}")
        #print(f"self.parameter_trees = {self.parameter_trees['system']}")
        #print(f"self.parameter_trees = {self.parameter_trees['system']['ROE'][0]}")

        the_list = self.parameter_trees[type][current_sub_tab_name];
        #list = current_sub_tab_widget.currentWidget()
        #print(f"the_list = {the_list}")
        #print(f"list = {list[0]}")
        #print(f"the_list = {the_list.selectedItems()}")
        #print(f"the_list = {the_list.objectName()}")

        for n in range(the_list.count()):
            item = the_list.item(n)
            #print(f"item = {item}")
            #id = item.id
            #print(f"item = {id}")
            #item.setFlags(Qt.NoItemFlags)
            item.enable()

        self.statusbar.showMessage('Cleared.')

        #for name in names:
            #print(f"name is {name}")
            ##for item in self.parameter_trees[type][name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
            #for item in self.parameter_trees[type][name]:
                #print(f"item is {item}")
                ##item.setCheckState(0, Qt.Unchecked)


    # https://doc.qt.io/qtforpython/PySide2/QtCore for findItems flags
    def deselectAllClicked(self, b):
        tab_name = b.objectName()
        current_tab = self.masterTabWidget.currentIndex()
        type = 'system' if current_tab == 0 else 'packets'
        for item in self.parameter_trees[type][tab_name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
            #if item.checkState(0) != Qt.Unchecked:
            item.setCheckState(0, Qt.Unchecked)
                #print(item.text(0) + " " + str(item.childCount()))
                # Feature in checkbox - doesnt set to unchecked if partially set. This doesn't clear it.


    def selectAllClicked(self, b):
        tab_name = b.objectName()
        current_tab = self.masterTabWidget.currentIndex()
        type = 'system' if current_tab == 0 else 'packets'
        #the_tree = self.parameter_trees[type][tab_name][0]
        #the_tree = self.parameter_trees[type][tab_name]
        for item in self.parameter_trees[type][tab_name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
            item.setCheckState(0, Qt.Checked)


    def saveFileDialog(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        self.output_filename, _ = QFileDialog.getSaveFileName(self, "QFileDialog.getSaveFileName()", self.output_dir, "csv files (*.csv)", options = options)
        if self.output_filename:
            print(f"File {self.output_filename} selected")
            suffix = QFileInfo(self.output_filename).suffix()
            if suffix != "csv":
                msg = QtWidgets.QMessageBox()
                msg.setIcon(QtWidgets.QMessageBox.Information)
                msg.setText("The file needs to end with .csv")
                msg.setDetailedText("The file will not be created or written to.")
                msg.setWindowTitle("Error")
                msg.setStandardButtons(QtWidgets.QMessageBox.Ok)
                retval = msg.exec_()
                self.output_filename = ''
            else:
                print(f"Using {self.output_filename}")
                self.output_dir = QFileInfo(self.output_filename).absoluteFilePath()
                pass
        else:
            print(f"No file selected")
            self.output_filename = ''

    # Rename to exportClicked
    def exportClicked(self):
        parray = []
        n_items = self.group_list_view.count()
        for i in range(n_items):
            gdict = {}
            pdict = {}
            parent = self.group_list_view.item(i).name
            group = self.groups[i]
#            print(f"Parent: {parent.name}")
#            print(f"Group : {group}")

#Parent: VIS_ROE_BIAS_DD
#Group : [<ListItem.ListItem object at 0x109555c18>, <ListItem.ListItem object at 0x109555ca8>, <ListItem.ListItem object at 0x109555d38>, <ListItem.ListItem object at 0x109555dc8>, <ListItem.ListItem object at 0x109555e58>, <ListItem.ListItem object at 0x109555ee8>, <ListItem.ListItem object at 0x109555f78>, <ListItem.ListItem object at 0x10955a048>, <ListItem.ListItem object at 0x10955a0d8>, <ListItem.ListItem object at 0x10955a168>, <ListItem.ListItem object at 0x10955a1f8>, <ListItem.ListItem object at 0x10955a288>]            

            unit = group[0].unit
            rate = group[0].src
            params = [item.name for item in group]

#            print(f"unit {unit}, rate {rate}, params {params}")

            pdict['unit'] = unit
            pdict['rate'] = rate
            pdict['params'] = params

            gdict[parent] = pdict
            parray.append(gdict)

#        for group in self.groups:
#            print(f"Group {group}")

#        print(json.dumps(self.group_list_view))

#        print(parray)
        print(json.dumps(parray))
        return


#self.group_list_view
#self.groups

        current_tab = self.masterTabWidget.currentIndex()
        type = 'system' if current_tab == 0 else 'packets'
        names = system_names if type == 'system' else packet_names
        items_checked = 0
        items = []
        for name in names:
            for item in self.parameter_trees[type][name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
                if (item.checkState(0) > 0):
                    if item.childCount() == 0:
                        items_checked += 1
                        items.append(item)
        if items_checked != 0:
            print(items)
            self.saveFileDialog()
            if self.output_filename == '':
                return
            f = open(self.output_filename, "w")
            for item in items:
                print (item.text(0),item.text(1), item.text(2), item.checkState(0))
                f.write(item.text(0) + ", " + item.text(1) + ", " + item.text(2) + "\n")
            f.close()
            #for name in names:
                #for item in self.parameter_trees[type][name].findItems("", Qt.MatchContains | Qt.MatchRecursive):
                    #if (item.checkState(0) > 0):
                        #if item.childCount() == 0:
                            #print (item.text(0),item.text(1), item.text(2), item.checkState(0))
        else:
            print("No items checked")
            msg = QtWidgets.QMessageBox()
            msg.setIcon(QtWidgets.QMessageBox.Information)
            msg.setText("No parameters are selected!")
            msg.setDetailedText("The file will not be created or written to.")
            msg.setWindowTitle("Error")
            msg.setStandardButtons(QtWidgets.QMessageBox.Ok)
            retval = msg.exec_()


    def cancelClicked(self):
        pass


    def createTab(self, type):
        font = QtGui.QFont()
        font.setPointSize(9)
        #font.setBold(True)
        font.setWeight(75)

        names = system_names if type == 'system' else packet_names
        tabs =  system_tabs  if type == 'system' else packet_tabs

        tabWidget = QtWidgets.QTabWidget()
        tabWidget.setFont(font)
        tabWidget.setObjectName("tabWidget")
        tab_number = 0
        for name in names:
            layout = QtWidgets.QHBoxLayout()
            tab = QtWidgets.QWidget()
            tab.setObjectName(name + '_tab')

            list = self.createList(name, tabs[name], type, tab_number)

            #self.parameter_trees[type][name].append(list)
            self.parameter_trees[type][name] = list

            #self.parameter_trees[type][name] = self.createTree(tab, name, tabs[name], type)
            
            layout.addWidget(list, stretch = 1)
            tab.setLayout(layout)

            tabWidget.addTab(tab, name)
            tab_number += 1

        tabWidget.setCurrentIndex(0)
        return tabWidget


    def createTabWidgetFrame(self):
        frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:lightgreen;")
        frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)

        #font = QtGui.QFont()
        #font.setPointSize(11)
        #font.setBold(True)
        #font.setWeight(75)

        self.masterTabWidget = QtWidgets.QTabWidget()
        #self.masterTabWidget.setStyleSheet("background-color:lightgreen;")
        #self.masterTabWidget.setFont(font)
        for type in choices:
            self.masterTabWidget.addTab(self.createTab(type), type)

        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.masterTabWidget)

        buttons_frame = QtWidgets.QFrame()
        buttons_frame_layout = QtWidgets.QHBoxLayout()
        buttons_frame_layout.setContentsMargins(0, 0, 0, 0)

        clearButton = QtWidgets.QPushButton("Clear")
        clearButton.clicked.connect(lambda:self.clearAllClicked(clearButton))

        groupButton = QtWidgets.QPushButton("Group")
        groupButton.clicked.connect(lambda:self.groupClicked(clearButton))

        buttons_frame_layout.addWidget(clearButton)
        buttons_frame_layout.addWidget(groupButton)
        buttons_frame_layout.addStretch(1)
        buttons_frame.setLayout(buttons_frame_layout)

        layout.addWidget(buttons_frame)

        frame.setLayout(layout)
        return frame

    """
    def createTree(self, tab, label, parameter_file, type):
        layout = QtWidgets.QVBoxLayout()
        tree = QtWidgets.QTreeWidget()
        tree.setColumnCount(3)
        tree.setHeaderLabels([label + " Parameters", "MIB ID", "Source"])

        tree.setSelectionMode(
            #QtWidgets.QAbstractItemView.MultiSelection |
            #QtWidgets.QAbstractItemView.ExtendedSelection |
            QtWidgets.QAbstractItemView.ContiguousSelection
            )

        # Yuck
        tree.setColumnWidth(0, 250)

        layout.addWidget(tree, stretch = 0)

        bottom_button_frame = QtWidgets.QFrame()
        deSelectAllButton = QtWidgets.QPushButton('Deselect All')
        deSelectAllButton.setObjectName(label)
        deSelectAllButton.clicked.connect(lambda:self.deselectAllClicked(deSelectAllButton))
        selectAllButton = QtWidgets.QPushButton('Select All')
        selectAllButton.setObjectName(label)
        selectAllButton.clicked.connect(lambda:self.selectAllClicked(selectAllButton))
        bottom_frame_layout = QtWidgets.QHBoxLayout()
        bottom_frame_layout.addWidget(deSelectAllButton)
        bottom_frame_layout.addWidget(selectAllButton)
        bottom_frame_layout.addStretch(1)
        bottom_button_frame.setLayout(bottom_frame_layout)
        layout.addWidget(bottom_button_frame, stretch = 0)

        tab.setLayout(layout)
        
        seen_parent = False
        blank = re.compile('^$')
        comment = re.compile('^\?')
        alt_comment = re.compile('^#')
        minus = re.compile('^---')
        pluses = re.compile('^\+\+\+')
        
        try:
            with open(parameter_file, 'r') as f:
                for line in f:
                    # txt
                    #items = line.split()
                    # csv
                    items = line.split(',')
                    if blank.match(line):
                        seen_parent = False
                        continue
                    if comment.match(line):
                        continue
                    if alt_comment.match(line):
                        continue
                    if minus.match(line):
                        seen_parent = True
                        parent = QtWidgets.QTreeWidgetItem(tree) 
                        parent.setText(0, items[1]) 
                        parent.setText(1, items[2])
                        parent.setText(2, items[3])
                        parent.setFlags(parent.flags() | Qt.ItemIsTristate | Qt.ItemIsUserCheckable) 
                        continue
                    if pluses.match(line):
                        seen_parent = True
                        parent = QtWidgets.QTreeWidgetItem(tree) 
                        parent.setText(0, items[1]) 
                        parent.setText(1, items[2])
                        parent.setText(2, items[3])
                        parent.setFlags(parent.flags() | Qt.ItemIsTristate | Qt.ItemIsUserCheckable) 
                        continue
                    if seen_parent:
                        child = QtWidgets.QTreeWidgetItem(parent) 
                        child.setFlags(child.flags() | Qt.ItemIsUserCheckable) 
                        child.setText(0, items[0]) 
                        child.setText(1, items[1])
                        child.setText(2, items[2])
                        child.setCheckState(0, Qt.Unchecked) 
                    else:
                        parent = QtWidgets.QTreeWidgetItem(tree) 
                        parent.setText(0, items[0]) 
                        parent.setText(1, items[1])
                        parent.setText(2, items[2])
                        parent.setFlags(parent.flags() | Qt.ItemIsUserCheckable) 
                        parent.setCheckState(0, Qt.Unchecked) 
                        seen_parent = False
                        
        except Exception as e:
            print("EXCEPTION")
        finally:
            f.close()

        return tree
        """

    def get_sub_tab_name(self):
        current_sub_tab_widget = self.masterTabWidget.currentWidget()
        current_sub_tab = current_sub_tab_widget.currentIndex()
        current_sub_tab_name = current_sub_tab_widget.tabText(current_sub_tab)
        #print(f"current_sub_tab_name = {current_sub_tab_name}, {self.masterTabWidget.currentIndex()}, {current_sub_tab}")
        return (current_sub_tab_name, current_sub_tab)


    def createList(self, label, parameter_file, type, main_tab_number):

        #list = MasterList.MasterList(label)
        list = List(label)
        list.resize(200,100)
        #list.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        list.setAcceptDrops(False)

        seen_parent = False
        blank = re.compile('^$')
        comment = re.compile('^\?')
        minus = re.compile('^---')
        pluses = re.compile('^\+\+\+')
        
        try:
            with open(parameter_file, 'r') as f:
                #print(f"Getting items for {label}")
                sub_tab_number = 0
                row = 0
                for line in f:
                    # txt
                    #items = line.split()
                    # csv
                    items = line.rstrip('\n').split(',')
                    #print(f"{items}")
                    if blank.match(line):
                        seen_parent = False
                        continue
                    if comment.match(line):
                        continue
                    if minus.match(line):
                        seen_parent = True
                        parent = ParentListItem(items[1], items[2], items[3], items[4], row)
                        list.addItem(parent)
                        row += 1
                        continue
                    if pluses.match(line):
                        seen_parent = True
                        parent = ParentListItem(items[1], items[2], items[3], items[4], row)
                        list.addItem(parent)
                        row += 1
                        continue
                    if seen_parent:
                        ###child = ListItem.ListItem(items[0], items[1], items[2], items[3], row)
                        child = ChildListItem(items[0], items[1], items[2], items[3], row)
                        child.main_tab = main_tab_number
                        child.sub_tab = sub_tab_number
                        child.set_parent(parent.name)
                        list.addItem(child)
#                        print(f"Adding child: {child.get_components()}, parent: {child.parent}")
                    else:
                        ###orphan = ListItem.ListItem(items[0], items[1], items[2], items[3], row)
                        orphan = ChildListItem(items[0], items[1], items[2], items[3], row)
                        orphan.main_tab = main_tab_number
                        orphan.sub_tab = sub_tab_number
                        list.addItem(orphan)
                        seen_parent = False
                        #print(f"Adding child: {orphan.get_components()}")
                    row += 1
                sub_tab_number += 1
        except Exception as e:
            print(f"EXCEPTION {e}")
        finally:
            #f.close()
            pass

        return list
        
        
if __name__ == '__main__': 

    app = QtWidgets.QApplication([])
    appWindow = chooserWindow()
    appWindow.show()
    sys.exit(app.exec_()) 
